
/*************************************************************************
 *
 *    Used with ICCARM and AARM.
 *
 *    (c) Copyright IAR Systems 2008
 *
 *    File name   : sd_ll_spi1.h
 *    Description : SD/MMC low level SPI0 driver
 *
 *    History :
 *    1. Date        : April 10, 2008
 *       Author      : Stanimir Bonev
 *       Description : Create
 *
 *    $Revision: 29 $
 **************************************************************************/
#include "sd_ll_spi0.h"

#include <NXP/iolpc2919.h>

/*************************************************************************
 * Function Name: SdChipSelect
 * Parameters: Boolean Select
 * Return: none
 *
 * Description: SD/MMC Chip select control
 * Select = true  - Chip is enable
 * Select = false - Chip is disable
 *
 *************************************************************************/
void SdChipSelect (Boolean Select)
{
  if (Select)
  {
    GPIO2_OR_bit.OR10 = 0;
  }
  else
  {
    // Synchronization
    while(SPI0_STATUS_bit.SPI_BUSY || !SPI0_STATUS_bit.RX_FIFO_EMPTY) {
	    if(!SPI0_STATUS_bit.RX_FIFO_EMPTY)
		    (void)SPI0_FIFO_DATA;
    }
    GPIO2_OR_bit.OR10 = 1;
  }
}
/*************************************************************************
 * Function Name: SdPresent
 * Parameters: none
 * Return: Boolean - true cart present
 *                 - false cart no present
 *
 * Description: SD/MMC precent check
 *
 *************************************************************************/
Boolean SdPresent (void)
{
  return !GPIO0_PINS_bit.PINS26;
}

/*************************************************************************
 * Function Name: SdWriteProtect
 * Parameters: none
 * Return: Boolean - true cart is protected
 *                 - false cart no protected
 *
 * Description: SD/MMC Write protect check
 *
 *************************************************************************/
Boolean SdWriteProtect (void)
{
  return !!GPIO0_PINS_bit.PINS27;
}

/*************************************************************************
 * Function Name: SdSetClockFreq
 * Parameters: Int32U Frequency
 * Return: Int32U
 *
 * Description: Set SPI ckl frequency
 *
 *************************************************************************/
Int32U SdSetClockFreq (Int32U Frequency)
{
  SPI0_CONFIG_bit.SPI_ENABLE = 0;

  SPI0_SLV0_SETTINGS1_bit.CLK_DIVISOR1 = 4*1000*1000 / 2 / Frequency;
  SPI0_SLV0_SETTINGS1_bit.CLK_DIVISOR2 = 2;
  SPI0_CONFIG_bit.SPI_ENABLE = 1;
  // Return real frequency
  return(Frequency);
}

/*************************************************************************
 * Function Name: SdInit
 * Parameters: none
 * Return: none
 *
 * Description: Init SPI, Cart Present, Write Protect and Chip select pins
 *
 *************************************************************************/
void SdInit (void)
{
  Int32U regVal;

  //init WP and CP!
  SFSP0_27 = (0x01<<2)|(0x00<<0);
  GPIO0_DR &= ~(0x1<<27);
  SFSP0_26 = (0x01<<2)|(0x00<<0);
  GPIO0_DR &= ~(0x1<<26);

  /* P2.10 is SPI0 SCS0, P2.11 is SPI0 SCLK, P2.12 is SPI0 SDI,
  P2.13 is SPI0 SDO */
  SFSP2_10 = (0x01<<2)|(0x00<<0);
  GPIO2_OR |= (0x1<<10);
  GPIO2_DR |= (0x1<<10);
  SFSP2_11 = (0x01<<2)|(0x03<<0);	/* Digital no PU and PD, func. 3. */
  SFSP2_12 = (0x01<<2)|(0x03<<0);	/* Digital no PU and PD, func. 3. */
  SFSP2_13 = (0x01<<2)|(0x03<<0);	/* Digital no PU and PD, func. 3. */

  /* Reset SPI0 */
  SPI0_CONFIG |= (0x1<<6);
  while ( SPI0_CONFIG & (0x1<<6) );	/* Self clear when reset is done. */
  SPI0_TX_FIFO_FLUSH = 1;			/* Flush TX FIFO */

  /* Default value, set to master mode, no loop back, normal mode, slave
  can drive its transmit-data output */
  regVal = SPI0_CONFIG;
  regVal &= ~((0x1<<1)|(0x1<<2)|(0x1<<3)|(0x1<<4));
  SPI0_CONFIG = regVal;

   /* Set slave of the SPI module */
  SPI0_SLV0_SETTINGS1 = (8) << 8 | (1);
  SPI0_SLV0_SETTINGS2 = (0x0<<9)|(0x1<<8)|0x07;	/* Motorola 8 bits, the rest is default */

  /* Slave 0 is enabled */
  SPI0_SLV_ENABLE |= (0x01<<0);

  /* Set SPI Update Enable bit */ 
  SPI0_CONFIG |= (0x1<<7);

  /* Setting SPI0 clock, for Atmel SEEPROM, SPI clock should be no more 
  than 3Mhz on 4.5V~5.5V, no more than 2.1Mhz on 2.7V~5.5V */
  SPI0_CONFIG |= (0x1<<0);		/* Enable SPI */
  return;
}

/*************************************************************************
 * Function Name: SdTranserByte
 * Parameters: Int8U ch
 * Return: Int8U
 *
 * Description: Read byte from SPI
 *
 *************************************************************************/
Int8U SdTranserByte (Int8U ch)
{
	while ( SPI0_STATUS_bit.SPI_BUSY );
	if ( SPI0_STATUS_bit.TX_FIFO_FULL )
	{
	  SPI0_TX_FIFO_FLUSH = 1;
	  while ( SPI0_STATUS_bit.SPI_BUSY );	/* Wait until flush is done */
	}	
  	SPI0_FIFO_DATA = ch;
  	while ( SPI0_STATUS_bit.SPI_BUSY );
  	while ( SPI0_STATUS_bit.RX_FIFO_EMPTY );
	ch = SPI0_FIFO_DATA;
	return ch; 
}

/*************************************************************************
 * Function Name: SdSendBlock
 * Parameters: pInt8U pData, Int32U Size
 *
 * Return: void
 *
 * Description: Read byte from SPI
 *
 *************************************************************************/
void SdSendBlock (pInt8U pData, Int32U Size)
{
  Int32U i;

  if ( Size == 0 )
	return;
  for ( i = 0; i < Size; i++ )
  {
	while (SPI0_STATUS_bit.SPI_BUSY );
	if ( SPI0_STATUS_bit.TX_FIFO_FULL )
	{
	  SPI0_TX_FIFO_FLUSH = 1;
	  while ( SPI0_STATUS_bit.SPI_BUSY );	/* Wait until flush is done */
	}	
  	SPI0_FIFO_DATA = *pData;
	pData++;
 	while (!SPI0_STATUS_bit.RX_FIFO_EMPTY )
		(void)SPI0_FIFO_DATA;
  }
  while ( SPI0_STATUS_bit.SPI_BUSY );
  while (!SPI0_STATUS_bit.RX_FIFO_EMPTY )
	  (void)SPI0_FIFO_DATA;

  return; 
}

/*************************************************************************
 * Function Name: SdReceiveBlock
 * Parameters: pInt8U pData, Int32U Size
 *
 * Return: void
 *
 * Description: Read byte from SPI
 *
 *************************************************************************/
void SdReceiveBlock (pInt8U pData, Int32U Size)
{
  Int32U i;

  for ( i = 0; i < Size; i++ )
  {
	/* wrtie dummy half-word out to generate clock, then read data from MISO */
	SPI0_FIFO_DATA = 0xFF;
	/* Wait for transfer complete, SPIF bit set */
	while (SPI0_STATUS_bit.RX_FIFO_EMPTY);
	*pData = SPI0_FIFO_DATA;
	while ( SPI0_STATUS_bit.SPI_BUSY );
	pData++;
  }
  return; 
}

/*************************************************************************
 * Function Name: SdDly_1ms
 * Parameters: Int32U Delay
 * Return: none
 *
 * Description: Delay [msec]
 *
 *************************************************************************/
void SdDly_1ms (Int32U Delay)
{
volatile Int32U i;
  for(;Delay;--Delay)
  {
    for(i = SD_DLY_1MSEC;i;--i);
  }
}
